/**
 * Sieve of Eratosthenes
 * Computes and prints the prime numbers up to MAXNUM.
 * This version uses an inefficient byte array to represent a set.
 */

#include <pbstdlib.h>

/* Maximal number. This number of bytes will be allocated
 * on stack, so the maximum cannot be much high. */
#define MAXNUM          32

int main()
{
    /* Sieve byte array. Each byte (element) represents 
     * one candidate number from 0 to MAXNUM.
     * Zero means candidate prime, one means we know it is NOT a prime.
     */
    uint8_t sieve[MAXNUM];
    
    /* clear the array */
    for (uint8_t i = 0; i < MAXNUM; ++i)
        sieve[i] = 0;
    
    /* start from 2, the first prime */
    for (uint8_t i = 2; ; ++i) {
        /* find the first prime in the sieve, i.e. the first
         * number without a flag set */
        for (; i < MAXNUM; ++i) {
            if (sieve[i] == 0) {
                /* it's prime;
                 * Print it to port 0 */
                __output(0, i);
                break;
            }
        }
        /* did we find any? */
        if (i == MAXNUM)
            /* no, stop */
            return 0;
        
        /* Mark all its multiplies up to MAXNUM not be a prime. */
        for (uint8_t k = 2*i; k < MAXNUM; k += i) {
            sieve[k] = 1;
        }
    }
    
    return 0;
}
