
# Use the well-known KCPSM3 by default
MARCH?=pblaze3
MCPU?=kcpsm3

# Default options: all warnings, max optimizations
CFLAGS?=-Wall -O3

# Location of the binaries; handy if the toolchain hasn't been installed yet
# in the $PATH, but the user is eager to try out the examples.
# Can be overriden on the make command line.
P?=$(SRCDIR)../../../bin/


A_OBJS=$(A_SRC:%.s=%.o)
C_ASMS=$(C_SRC:%.c=%.s)
C_OBJS=$(C_SRC:%.c=%.o)
ELF=$(FINAL).elf
DIS=$(FINAL).dis
BIN=$(FINAL).bin
INC=$(FINAL).inc
MEM=$(FINAL).mem

RESULTS=$(A_OBJS) $(C_ASMS) $(C_OBJS) \
	$(ELF) $(DIS) $(BIN) $(INC) $(MEM)


all : $(RESULTS)
	@echo "Settings: MCPU="$(MCPU)

clean :
	rm -f $(RESULTS)

# #################################################################
# NOTE: the $(SRCDIR) is needed to handle the case when the sources
# are in a different directory

# Compile .c to assembly .s
%.s : $(SRCDIR)%.c
	$(P)pblaze-cc -march=$(MARCH) -mcpu=$(MCPU) -c -S $(CFLAGS) $< -o $@

# Assemble .s to objects .o
%.o : %.s
	$(P)pblaze-cc -march=$(MARCH) -mcpu=$(MCPU) -c $< -o $@

%.o : $(SRCDIR)%.s
	$(P)pblaze-cc -march=$(MARCH) -mcpu=$(MCPU) -c $< -o $@

# Link all .o objects to a program .elf
$(ELF) : $(A_OBJS) $(C_OBJS)
	$(P)pblaze-cc -march=$(MARCH) -mcpu=$(MCPU) $^ -o $@

# Disassemble .elf to .dis
%.dis : %.elf
	$(P)pblaze-readelf -a $^  >  $@
	$(P)pblaze-objdump -d $<  >> $@

# Extract text from an .elf to a flat binary image .bin,
# and check that text limits (max 1024 instructions ~~ 4kB)
%.bin : %.elf
	objcopy -I elf32-little -O binary -j .text $^ $@
	@if [ `stat -c%s $@` -ge 4096 ]; then echo -e "\e[0;31mERROR: Firmware image file $@ is bigger than 1024 words (4kB)!!\e[00m"; fi

# Convert binary image .bin to a C header for inclusion in a microblaze program
%.inc : %.bin
	$(P)pblaze-bincopy --format=c $< $@

# Convert binary image .bin to a MEM file that can be loaded into a vhdl-based simulator
%.mem : %.bin
	$(P)pblaze-bincopy --format=mem $< $@
