
#ifndef PBSTDLIB_H
#define PBSTDLIB_H

#ifndef __HOSTSIM__
/** ===================================================================================== */

/** integer types, normally in <stdint.h> */

typedef unsigned char uint8_t;
typedef unsigned int uint16_t;

typedef signed char int8_t;
typedef signed int int16_t;


/** ===================================================================================== */
/**
 * Input value from the port.
 */
static inline uint8_t __input(uint8_t port)
{
    uint8_t result;
    asm volatile (
        "input %0, %1"          // %0 = result, %1 = port
        : /* outputs */
            /*0*/ "=r" (result)
        : /* inputs */
            /*1*/ "ir" (port)
        : /* clobbers */
    );
    return result;
}

/**
 * Output value to a port.
 */
static inline void __output(uint8_t port, uint8_t value)
{
    asm volatile (
        "output %0, %1"         // %0 = value, %1 = port
        : /* outputs */
        : /* inputs */
            /*0*/ "r" (value),
            /*1*/ "ir" (port)
        : /* clobbers */
    );
}

/* this stops the simulation by writing 0xFF to port 0xFF */
static inline void stop()
{
    __output(0xFF, 0xFF);
    /* loop indefinitely */
    for (;;) ;
}

/* print the value of i on port 0x00 */
static inline void print(uint8_t i)
{
    __output(0x00, i);
}

/* print the value of i on port ch */
static inline void printch(uint8_t ch, uint8_t i)
{
    __output(ch, i);
}

#else

/** ===================================================================================== */
/** host simulation; only headers here */
#include <stdint.h>

void stop();
void print(uint8_t i);
void printch(uint8_t ch, uint8_t i);
void __output(uint8_t port, uint8_t value);
uint8_t __input(uint8_t port);

/** ===================================================================================== */
#endif

/* intrinsic function: 8b multiplication */
extern int8_t __pblibc_mul_i8(int8_t a, int8_t b);

/* intrinsic function: 16b multiplication */
extern int16_t __pblibc_mul_i16(int16_t a, int16_t b);

#endif
